<?php

namespace Qls\QlsServicePoints\Observer;

use Magento\Framework\Event\Observer as EventObserver;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Quote\Model\QuoteRepository;

use Magento\Sales\Model\Order;
use Qls\QlsServicePoints\Model\QLSServicePointValidator as Validator;

class SalesModelServiceQuoteSubmitBefore implements ObserverInterface
{
    /**
     * @var QuoteRepository
     */
    private $quoteRepository;

    /**
     * @var Validator
     */
    private $validator;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $state;

    /**
     * SalesModelServiceQuoteSubmitBefore constructor.
     *
     * @param QuoteRepository $quoteRepository
     * @param Validator $validator
     */
    public function __construct(
        QuoteRepository $quoteRepository,
        Validator $validator,
        \Magento\Framework\App\State $state,
        \Psr\Log\LoggerInterface $logger
    )
    {
        $this->quoteRepository = $quoteRepository;
        $this->logger = $logger;
        $this->validator = $validator;
        $this->state = $state;
    }

    /**
     * @param EventObserver $observer
     * @return $this
     * @throws \Exception
     */
    public function execute(EventObserver $observer)
    {
        $this->logger->debug(__METHOD__);

        /** @var Order $order */
        $order = $observer->getOrder();
        /** @var \Magento\Quote\Model\Quote $quote */
        $quote = $this->quoteRepository->get($order->getQuoteId());

        $areaCode = $this->state->getAreaCode();

        $this->logger->debug(__METHOD__ . ' area : ' . $areaCode);

        // do not proceed with the validation in the Admin area
        if ($areaCode !== \Magento\Framework\App\Area::AREA_ADMINHTML) {
            if (!$this->validator->validate($quote)) {
                throw new LocalizedException(__('Please select the ServicePoint first.'));
            }
        }

        $order->setQlsServicepointCode($quote->getQlsServicepointCode());

        return $this;
    }
}
